@@ -6,32 +6,36 @@ const ZERO: &str = "a0";
 /// Either a or b can be empty strings. If a is empty it indicates smallest key,
 /// If b is empty it indicates largest key.
 /// b must be empty string or > a.
-pub fn key_between(a: &str, b: &str) -> Result<String, String> {
+pub fn key_between(a: &Option<String>, b: &Option<String>) -> Result<String, String> {
   // println!("between: {} {}", a, b);
-  if !a.is_empty() {
-    validate_order_key(a)?;
+  if !a.is_none() {
+    validate_order_key(a.as_ref().unwrap())?;
   }
-  if !b.is_empty() {
-    validate_order_key(b)?;
+  if !b.is_none() {
+    validate_order_key(b.as_ref().unwrap())?;
   }
-  if !a.is_empty() && !b.is_empty() && a >= b {
-    return Err(format!("invalid order: {} >= {}", a, b));
+  if !a.is_none() && !b.is_none() && a >= b {
+    return Err(format!(
+      "invalid order: {} >= {}",
+      a.as_deref().unwrap(),
+      b.as_deref().unwrap()
+    ));
   }
-  if a.is_empty() {
-    if b.is_empty() {
+  if a.is_none() {
+    if b.is_none() {
       return Ok(ZERO.to_owned());
     }
 
-    let ib = get_int_part(b)?;
+    let int_b = get_int_part(b.as_deref().unwrap())?;
 
-    let fb = &b[ib.len()..];
-    if ib == SMALLEST_INT {
-      return Ok((ib as String) + &midpoint("", fb));
+    let float_part_b = &b.as_deref().unwrap()[int_b.len()..];
+    if int_b == SMALLEST_INT {
+      return Ok((int_b as String) + &midpoint("", float_part_b));
     }
-    if ib.as_str() < b {
-      return Ok(ib);
+    if int_b.as_str() < b.as_deref().unwrap() {
+      return Ok(int_b);
     }
-    let res = decrement_int(&ib)?;
+    let res = decrement_int(&int_b)?;
 
     if res.is_empty() {
       return Err("range underflow".to_owned());
@@ -39,49 +43,48 @@ pub fn key_between(a: &str, b: &str) -> Result<String, String> {
     return Ok(res);
   }
 
-  if b.is_empty() {
-    let ia = get_int_part(a)?;
+  if b.is_none() {
+    let int_a = get_int_part(a.as_deref().unwrap())?;
 
-    let fa = &a[ia.len()..];
-    let i = increment_int(&ia)?;
+    let float_part_a = &a.as_deref().unwrap()[int_a.len()..];
+    let i = increment_int(&int_a)?;
     if i.is_empty() {
-      return Ok(ia + &midpoint(fa, ""));
+      return Ok(int_a + &midpoint(float_part_a, ""));
     }
     return Ok(i);
   }
 
-  let ia = get_int_part(a)?;
+  let int_a = get_int_part(a.as_deref().unwrap())?;
 
-  let fa = &a[ia.len()..];
-  let ib = get_int_part(b)?;
+  let float_part_a = &a.as_deref().unwrap()[int_a.len()..];
+  let int_b = get_int_part(b.as_ref().unwrap())?;
 
-  let fb = &b[ib.len()..];
-  if ia == ib {
-    return Ok(ia + &midpoint(fa, fb));
+  let float_part_b = &b.as_ref().unwrap()[int_b.len()..];
+  if int_a == int_b {
+    return Ok(int_a + &midpoint(float_part_a, float_part_b));
   }
-  let i = increment_int(&ia)?;
+  let i = increment_int(&int_a)?;
 
   if i.is_empty() {
     return Err("range overflow".to_owned());
   }
-  if i.as_str() < b {
+  if i.as_str() < b.as_ref().unwrap() {
     return Ok(i);
   }
-  Ok(ia + &midpoint(fa, ""))
+  Ok(int_a + &midpoint(float_part_a, ""))
 }
 
 /// `a < b` lexicographically if `b` is non-empty.
 /// a == "" means first possible string.
 /// b == "" means last possible string.
+/// a, b MUST be str without head
 fn midpoint(a: &str, b: &str) -> String {
   if !b.is_empty() {
     // remove longest common prefix.  pad `a` with 0s as we
     // go.  note that we don't need to pad `b`, because it can't
     // end before `a` while traversing the common prefix.
     let mut i = 0;
-    // TODO how Go made this happen?
-    // https://github.com/rocicorp/fracdex/blob/main/fracdex.go#L110-L112
-    for _ in 0..std::cmp::max(a.len(), b.len()) {
+    for _ in 0..a.len() {
       let mut c: char = '0';
       if a.len() > i {
         c = a.chars().nth(i).unwrap()
@@ -93,9 +96,9 @@ fn midpoint(a: &str, b: &str) -> String {
     }
     if i > 0 {
       if i as i64 > a.len() as i64 - 1 {
-        return b[0..i].to_string() + midpoint("", &b[i..]).as_str();
+        return b[0..i].to_string() + &midpoint("", &b[i..]);
       } else {
-        return b[0..i].to_string() + midpoint(&a[i..], &b[i..]).as_str();
+        return b[0..i].to_string() + &midpoint(&a[i..], &b[i..]);
       }
     }
   }
@@ -116,7 +119,10 @@ fn midpoint(a: &str, b: &str) -> String {
 
   // first digits are consecutive
   if b.len() > 1 {
-    return b[0..1].to_string();
+    if !b.starts_with('0') {
+      return b[0..1].to_string();
+    }
+    return BASE62_DIGITS.chars().nth(digit_a).unwrap().to_string() + &midpoint("", &b[1..]);
   }
 
   // `b` is empty or has length 1 (a single digit).
@@ -141,6 +147,11 @@ fn validate_int(i: &str) -> Result<(), String> {
   Ok(())
 }
 
+/**
+ * length map:
+ * A-Z -> 28-2
+ * a-z -> 2-28
+ */
 fn get_int_len(head: char) -> Result<usize, String> {
   if ('a'..='z').contains(&head) {
     Ok((head as usize - 'a' as usize + 2) as usize)
@@ -151,6 +162,9 @@ fn get_int_len(head: char) -> Result<usize, String> {
   }
 }
 
+/**
+ * throw error when shorter than `get_int_len(head)`
+ */
 fn get_int_part(key: &str) -> Result<String, String> {
   let int_part_len = get_int_len(key.chars().next().unwrap())?;
 
@@ -160,6 +174,12 @@ fn get_int_part(key: &str) -> Result<String, String> {
   Ok(key[0..int_part_len].to_string())
 }
 
+/**
+ * throw when:
+ * first charater is not valid head
+ * short than `get_int_len(head)`
+ * ends with 0
+ */
 fn validate_order_key(key: &str) -> Result<(), String> {
   if key == SMALLEST_INT {
     return Err(format!("invalid order key: {}", key));
@@ -167,108 +187,96 @@ fn validate_order_key(key: &str) -> Result<(), String> {
   // get_int_part will return error if the first character is bad,
   // or the key is too short.  we'd call it to check these things
   // even if we didn't need the result
-  let i = get_int_part(key)?;
+  let int_part = get_int_part(key)?;
 
-  let f = &key[i.len()..];
-  if f.ends_with('0') {
+  let float_part = &key[int_part.len()..];
+  if float_part.ends_with('0') {
     return Err(format!("invalid order key: {}", key));
   }
   Ok(())
 }
 
 /// returns error if x is invalid, or if range is exceeded
+/// x MUST be int without float part
 fn increment_int(x: &str) -> Result<String, String> {
   validate_int(x)?;
 
-  let mut digs: Vec<String> = x
-    .split("")
-    .filter(|&x| !x.is_empty())
-    .collect::<Vec<&str>>()
-    .iter()
-    .map(|s| s.to_string())
-    .collect();
-  let head = digs[0].to_owned();
+  let mut digs: Vec<char> = x.chars().collect();
+  let head = digs[0];
   digs.remove(0);
   let mut carry = true;
 
   let mut i = digs.len() as i64 - 1;
   while carry && i >= 0 {
-    let d = BASE62_DIGITS.find(&digs[i as usize]).unwrap() + 1;
+    let d = BASE62_DIGITS.find(digs[i as usize]).unwrap() + 1;
     if d == BASE62_DIGITS.len() {
-      digs[i as usize] = "0".to_owned()
+      digs[i as usize] = '0';
     } else {
-      digs[i as usize] = BASE62_DIGITS.chars().nth(d).unwrap().to_string();
+      digs[i as usize] = BASE62_DIGITS.chars().nth(d).unwrap();
       carry = false;
     }
 
     i -= 1;
   }
   if carry {
-    if head == "Z" {
+    if head == 'Z' {
       return Ok("a0".to_owned());
     }
-    if head == "z" {
+    if head == 'z' {
       return Ok("".to_owned());
     }
-    let h = ((head.chars().next().unwrap() as u8 + 1) as char).to_string();
-    if h.as_str() > "a" {
-      digs.push("0".to_owned())
+    let h = (head as u8 + 1) as char;
+    if h > 'a' {
+      // a-z -> incr
+      digs.push('0')
     } else {
-      digs.remove(0);
+      // A-Z -> decr
+      digs.pop();
     }
-    return Ok((h as String) + &digs.join(""));
+    return Ok(h.to_string() + &digs.iter().collect::<String>());
   }
-  Ok(head.to_owned() + &digs.join(""))
+  Ok(head.to_string() + &digs.iter().collect::<String>())
 }
 
 fn decrement_int(x: &str) -> Result<String, String> {
   validate_int(x)?;
 
-  let mut digs: Vec<String> = x
-    .split("")
-    .filter(|&x| !x.is_empty())
-    .collect::<Vec<_>>()
-    .iter()
-    .map(|s| s.to_string())
-    .collect();
+  let mut digs: Vec<char> = x.chars().collect();
 
-  let head = digs[0].to_owned();
+  let head = digs[0];
   digs.remove(0);
   let mut borrow = true;
 
   let mut i = digs.len() as i64 - 1;
   while borrow && i >= 0 {
-    let d: i64 = match BASE62_DIGITS.find(&digs[i as usize]) {
-      Some(n) => (n as i64 - 1),
-      None => -2, // TODO
-    };
+    let d: i64 = BASE62_DIGITS.find(digs[i as usize]).unwrap() as i64 - 1;
 
     if d == -1 {
-      digs[i as usize] = BASE62_DIGITS.chars().nth_back(0).unwrap().to_string();
+      digs[i as usize] = BASE62_DIGITS.chars().nth_back(0).unwrap();
     } else {
-      digs[i as usize] = BASE62_DIGITS.chars().nth(d as usize).unwrap().to_string();
+      digs[i as usize] = BASE62_DIGITS.chars().nth(d as usize).unwrap();
       borrow = false
     }
     i -= 1;
   }
 
   if borrow {
-    if head == "a" {
+    if head == 'a' {
       return Ok("Z".to_owned() + &BASE62_DIGITS.chars().nth_back(0).unwrap().to_string());
     }
-    if head == "A" {
+    if head == 'A' {
       return Ok("".to_owned());
     }
-    let h: char = (head.chars().next().unwrap() as u8 - 1) as char;
+    let h: char = (head as u8 - 1) as char;
     if h < 'Z' {
-      digs.push((BASE62_DIGITS.chars().nth_back(0).unwrap()).to_string());
+      digs.push(BASE62_DIGITS.chars().nth_back(0).unwrap());
     } else {
-      digs.remove(0);
+      digs.pop();
     }
-    return Ok(h.to_string() + &digs.join(""));
+    return Ok(h.to_string() + &digs.iter().collect::<String>());
   }
 
-  Ok(head.to_owned() + &digs.join(""))
+  Ok(head.to_string() + &digs.iter().collect::<String>())
 }
 
 /// float64_approx converts a key as generated by key_between() to a float64.
@@ -284,7 +292,7 @@ pub fn float64_approx(key: &str) -> Result<f64, String> {
 
   let ip = get_int_part(key)?;
 
-  let mut digs: Vec<&str> = ip.split("").filter(|&x| !x.is_empty()).collect::<Vec<_>>();
+  let mut digs: Vec<char> = ip.chars().collect();
   let head = digs[0];
   digs.remove(0);
   let mut rv: f64 = 0.0;
@@ -306,7 +314,7 @@ pub fn float64_approx(key: &str) -> Result<f64, String> {
     rv += (p.unwrap() as f64) / (BASE62_DIGITS.len() as f64).powf((i + 1) as f64)
   }
 
-  if head < "a" {
+  if head < 'a' {
     rv *= -1.0;
   }
 
@@ -317,7 +325,7 @@ pub fn float64_approx(key: &str) -> Result<f64, String> {
 /// Either a or b can be empty strings. If a is empty it indicates smallest key,
 /// If b is empty it indicates largest key.
 /// b must be empty string or > a.
-pub fn n_keys_between(a: &str, b: &str, n: usize) -> Result<Vec<String>, String> {
+pub fn n_keys_between(a: &Option<String>, b: &Option<String>, n: usize) -> Result<Vec<String>, String> {
   if n == 0 {
     return Ok(vec![]);
   }
@@ -326,25 +334,25 @@ pub fn n_keys_between(a: &str, b: &str, n: usize) -> Result<Vec<String>, String>
 
     return Ok(vec![c]);
   }
-  if b.is_empty() {
+  if b.is_none() {
     let mut c = key_between(a, b)?;
     let mut result: Vec<String> = Vec::with_capacity(n);
     result.push(c.to_owned());
 
     for _i in 0..((n as usize) - 1) {
-      c = key_between(&c, b)?;
+      c = key_between(&Some(c), b)?;
       result.push(c.to_owned());
     }
 
     return Ok(result);
   }
-  if a.is_empty() {
+  if a.is_none() {
     let mut c = key_between(a, b)?;
 
     let mut result: Vec<String> = Vec::with_capacity(n);
     result.push(c.to_owned());
     for _i in 0..(n as usize - 1) {
-      c = key_between(a, &c)?;
+      c = key_between(a, &Some(c))?;
       result.push(c.to_owned());
     }
     result.reverse();
@@ -355,14 +363,14 @@ pub fn n_keys_between(a: &str, b: &str, n: usize) -> Result<Vec<String>, String>
 
   let mut result: Vec<String> = Vec::with_capacity(n);
   {
-    let key_r = n_keys_between(a, &c, mid)?;
+    let key_r = n_keys_between(a, &Some(c.to_owned()), mid)?;
     for item in key_r {
       result.push(item.clone());
     }
   }
   result.push(c.to_owned());
   {
-    let key_r = n_keys_between(&c, b, n - mid - 1)?;
+    let key_r = n_keys_between(&Some(c), b, n - mid - 1)?;
     for item in key_r.iter() {
       result.push(item.to_owned());
     }
